import { forwardRef, useState } from "react";
import styled from "styled-components";
import { isNull } from "../utils/type-util";
import { isValidColorHex, rgbStringToHex } from "../utils/color-util";
import { useDisclosure, useControllableState, useForkHandler } from "../hooks";
import { EyeDropperIcon } from "../icons";
import { KeyCode } from "../utils/key-code";
import { Input } from "./input";
import { Dropdown } from "./dropdown";
import { ColorPicker } from "./color-picker";
import { ColorSwatch } from "./color-swatch";
import { jsx as _jsx } from "react/jsx-runtime";
import { jsxs as _jsxs } from "react/jsx-runtime";
function escapeInput(val) {
  return val.replace(/([^0-9A-F]+)/gi, "").substr(0, 6);
}
function isEyeDropperSupported() {
  return "EyeDropper" in window;
}
async function openEyeDropper() {
  // @ts-ignore
  const {
    sRGBHex: color
  } = await new window.EyeDropper().open();
  return color.startsWith("rgb") ? rgbStringToHex(color) : color;
}
const StyledSuffix = styled(Input.Suffix).withConfig({
  displayName: "color-input__StyledSuffix",
  componentId: "sc-1psc2tg-0"
})(["color:", ";pointer-events:auto;cursor:pointer;"], p => p.theme.colors.text);
const StyledGroup = styled(Input.Group).withConfig({
  displayName: "color-input__StyledGroup",
  componentId: "sc-1psc2tg-1"
})(["input:disabled ~ ", "{opacity:0.5;cursor:not-allowed;}"], StyledSuffix);
export const ColorInput = /*#__PURE__*/forwardRef((_ref, ref) => {
  let {
    value,
    defaultValue = "#000000",
    onChange,
    onOpen,
    onClose,
    onBlur,
    onKeyDown,
    pickerSize,
    pickerDisabled,
    disabled,
    className,
    ...rest
  } = _ref;
  const {
    isOpen,
    open,
    close
  } = useDisclosure({
    onOpen,
    onClose
  });
  const [color, setColor] = useControllableState(value, defaultValue);
  const [inputValue, setInputValue] = useState(null);
  const handleInputChange = val => {
    const escVal = escapeInput(val);
    setInputValue(escVal);
    if (isValidColorHex(escVal)) {
      const hex = "#" + escVal;
      setColor(hex);
      onChange?.(hex);
    }
  };
  const handleInputBlur = event => {
    setInputValue(null);
    onBlur?.(event);
  };
  const handlePickerChange = val => {
    setColor(val);
    onChange?.(val);
  };
  const handleEyeDropper = event => {
    event.preventDefault();
    if (!disabled) {
      openEyeDropper().then(val => {
        setColor(val);
        onChange?.(val);
      });
    }
  };
  const handleKeyDown = event => {
    if (event.key === KeyCode.ENTER && !pickerDisabled) {
      open();
    } else if (event.key === KeyCode.ESC) {
      event.stopPropagation();
      close();
    }
  };
  const inputProps = {
    onKeyDown: useForkHandler(onKeyDown, handleKeyDown),
    disabled,
    ...rest
  };
  const overlay = /*#__PURE__*/_jsx(ColorPicker, {
    color: color,
    onChange: handlePickerChange,
    size: pickerSize
  });
  return /*#__PURE__*/_jsx(Dropdown, {
    overlay: overlay,
    isOpen: isOpen,
    onOpen: open,
    onClose: close,
    disabled: pickerDisabled || disabled,
    keepMounted: false,
    matchWidth: false,
    p: 2,
    children: /*#__PURE__*/_jsxs(StyledGroup, {
      className: className,
      children: [/*#__PURE__*/_jsx(Input.Prefix, {
        children: /*#__PURE__*/_jsx(ColorSwatch, {
          color: color,
          size: 14
        })
      }), /*#__PURE__*/_jsx(Input, {
        ref: ref,
        value: isNull(inputValue) ? color : "#" + inputValue,
        onChange: handleInputChange,
        onBlur: handleInputBlur,
        spellCheck: "false",
        textTransform: "uppercase",
        ...inputProps
      }), isEyeDropperSupported() ? /*#__PURE__*/_jsx(StyledSuffix, {
        onClick: handleEyeDropper,
        children: /*#__PURE__*/_jsx(EyeDropperIcon, {})
      }) : null]
    })
  });
});
if (process.env.NODE_ENV !== "production") {
  ColorInput.displayName = "ColorInput";
}